#include <sdcc-lib.h>
#include <pic12f675.h>	// /usr/share/sdcc/non-free/include/pic14/pic12f675.h

#include "inline.h"
#include "libwait.h"

__code unsigned char __at 0x2000 id_location[] = {'A', 'B', 'C', 'D'};

unsigned int __at 0x2007 __config =
	_CPD_OFF &
	_CP_OFF &
	_BODEN_ON &	// for 2.1V
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

// EEPROM へ記録するアドレス(0x7C: 0x04)
__code unsigned char __at 0x2100 eedata[] = { 0x7C, 0, 0, 0 };

void Intr(void) __interrupt 0 {
	T0IF = 0;						// Clear timer interrupt
}

/*			  _______
	Vcc		-|	 U	 |-	GND
	LED0	-|5		0|-	LED3
	CdS		-|4		1|-	LED2
	NC		-|3_____2|-	LED1		// Analog: 0, 1, 2, 4
*/

unsigned char level_to_ledpat(unsigned char level);
void blink_ledpat(unsigned char ledpat, unsigned char times);

unsigned char eeread(unsigned char r_adr);
void eewrite(unsigned char w_adr, unsigned char w_data);

void main(void) {

	unsigned char cds, level;
	unsigned char w_adr;

				//  --543210
	GPIO		= 0b00000000;		// GPIO3: input only
	CMCON		= 0b00000111;
	ANSEL		= 0b00011000;
	TRISIO		= 0b00011000;		// 1: 入力, 0: 出力

	ADCON0		= 0b00001101;		// 左揃え, 電源電圧, AN3, DONE, ON

	OPTION_REG	= 0b11111111;		// 初期値:0b11111111
									// 7:GPIOプルアップ(0:使用する)
									// 6:GP2割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(011:1/16)
									// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz

	INTCON		= 0b00000000;		// 初期値:0b00000000
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE GP2/INT External Interrupt Enable bit
									// 3:GPIE GPIO Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF GP2/INT External Interrupt Flag bit
									// 0:GPIF GPIO Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:ADIE A/D Converter Interrupt Enable bit
									// 3:CMIE Comparator Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 6:TMR1GE Timer1 Gate Enable bit
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable Control bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Control bit(1:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

//	forCalibOsc();

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//

	// EEPROM へ記録するアドレスを呼び出す
	if(w_adr = eeread(0))						// 0x7C -> 0x78 -> ... -> 0x04 -> 0x00
		eewrite(0, w_adr - 4);

	while(1) {

		// 全点灯
		blink_ledpat(level_to_ledpat(15), 5);	// 1 sec

		// CdS の A/D 変換
		GO = 1;
		while(NOT_DONE);
		cds = ADRESH;

		// CdS の A/D 変換値を EEPROM に保存(校正用)
		if(w_adr) {
			eewrite(--w_adr ^ 0x7F, cds);		// 0x04 -> 0x08 -> ... -> 0x7C -> ----
			if(!(w_adr & 0x3))
				w_adr = 0;
		}

		// CdS の A/D 変換値を level(EV - 6) に変換
		level = 0;								//  0: EV  6, f4      1/4
		if(cds >  15) level++;					//  1: EV  7, f4      1/8
		if(cds >  23) level++;					//  2: EV  8, f4     1/15
		if(cds >  36) level++;					//  3: EV  9, f4     1/30
		if(cds >  55) level++;					//  4: EV 10, f4     1/60
		if(cds >  75) level++;					//  5: EV 11, f4    1/125
		if(cds >  94) level++;					//  6: EV 12, f4    1/250, 曇・日陰
		// 以下、適当(未校正)
		if(cds > 113) level++;					//  7: EV 13, f5.6  1/250, 明るい曇
		if(cds > 132) level++;					//  8: EV 14, f8    1/250, 晴
		if(cds > 151) level++;					//  9: EV 15, f11   1/250, 快晴
		if(cds > 170) level++;					// 10: EV 16, f16   1/250, 快晴時の海・山・雪景色
		if(cds > 189) level++;					// 11: EV 17, f16   1/500
		if(cds > 208) level++;					// 12: EV 18, f16  1/1000
		if(cds > 227) level++;					// 13: EV 19, f16  1/2000
		if(cds > 246) level++;					// 14: EV 20, f16  1/4000
		if(cds > 254) level++;					// 15: EV 21, f16  1/8000

		// level 表示
		blink_ledpat(level_to_ledpat(level), 5);	// 1 sec

		// CdS の A/D 変換値表示(校正用)
		blink_ledpat(level_to_ledpat(cds >> 4), 5);	// 1 sec
		blink_ledpat(level_to_ledpat(cds), 5);		// 1 sec

		// 消灯
		blink_ledpat(level_to_ledpat(0), 5);		// 1 sec
	}
}

unsigned char level_to_ledpat(unsigned char level) {

	unsigned char result = 0;

	if(level & 0b0001) result |= 0b100000;
	if(level & 0b0010) result |= 0b000100;
	if(level & 0b0100) result |= 0b000010;
	if(level & 0b1000) result |= 0b000001;

	return(result);
}

void blink_ledpat(unsigned char ledpat, unsigned char times) {

	unsigned char i;

	for(i = 0; i < times; i++) {
		GPIO = ledpat;
		waitL(1);								// 0.1 sec
		GPIO = 0;
		waitL(1);								// 0.1 sec
	}
}

unsigned char eeread(unsigned char r_adr) {

	EEADR = r_adr;
	RD = 1;

	return(EEDATA);
}

void eewrite(unsigned char w_adr, unsigned char w_data) {

	EEADR = w_adr;
	EEDATA = w_data;

	WREN = 1;
	GIE = 0;

	EECON2 = 0x55;
	EECON2 = 0xAA;
	WR = 1;

	GIE = 1;
	WREN = 0;
}

